<?php

namespace Drupal\ww_publish\Event;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\user\Entity\User;
use Drupal\ww_publish\Entity\SnsMessageEntity;
use Drupal\ww_publish\Job;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\amazon_sns\Event\SnsEvents as AmazonSnsEvents;
use Drupal\amazon_sns\Event\SnsMessageEvent as AmazonSnsMessageEvent;

/**
 * Class SnsNotificationSubscriber.
 *
 * Subscribe to SNS notification events. These are in response to events
 * dispatched by NotificationController::receive(). This file is based on
 * \Drupal\amazon_sns\Event\SnsNotificationSubscriber.
 *
 * @package Drupal\ww_publish\Event
 */
class SnsNotificationSubscriber implements ContainerInjectionInterface, EventSubscriberInterface {

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  private $logger;

  /**
   * Configuration of the ww_publish module.
   *
   * @var \Drupal\Core\Config\Config
   */
  private $config;

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      AmazonSnsEvents::NOTIFICATION => 'processSnsMessageEvent',
    ];
  }

  /**
   * SnsNotificationSubscriber constructor.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   A config factory.
   */
  public function __construct(LoggerChannelFactoryInterface $logger_factory, ConfigFactoryInterface $config_factory) {
    $this->logger = $logger_factory->get('ww_publish');
    $this->config = $config_factory->getEditable('ww_publish.settings');
    // Set user for logger.
    if ($userId = $this->config->get('ww_user')) {
      $currentUser = User::load($userId);
      $this->logger->setCurrentUser($currentUser);
    }
  }

  /**
   * {@inheritdoc}
   *
   * @codeCoverageIgnore
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('logger.factory'),
      $container->get('config.factory')
    );
  }

  /**
   * Process the SNS message event.
   *
   * @param \Drupal\amazon_sns\Event\SnsMessageEvent $event
   *   Publish message received from Amazon SNS.
   *
   * @return bool
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function processSnsMessageEvent(AmazonSnsMessageEvent $event) {
    $eventMessageRaw = $event->getMessage();

    if (isset($eventMessageRaw['TopicArn']) && strpos($eventMessageRaw['TopicArn'], 'ecs-export-topic' ) > 0 && isset($eventMessageRaw['Message'])) {
      $eventMessage = json_decode($eventMessageRaw['Message']);
      if ($this->config->get('debug_mode'))
        $this->logger->info('Amazon SNS message event: <pre><code>' . print_r($eventMessage, TRUE) . '</code></pre>');

      if ($this->config->get('cron_mode')) {
        // Save the SNS message as a configuration entity.
        $newSnsMessage = [
          'id' => $eventMessage->id,
          'name' => $eventMessage->name,
          'url' => $eventMessage->url,
          'metadata_url' => $eventMessage->metadataUrl,
          'article_json_url' => $eventMessage->articleJsonUrl,
          'tenant_id' => $eventMessage->tenantId,
          'brand' => $eventMessage->brand,
        ];
        $snsMessageEntity = new SnsMessageEntity($newSnsMessage, 'ww_publish_sns_message');
        $snsMessageEntity->save();
      } else {
        $job = new Job($this->config, $this->logger);
        $job->publishArticle($eventMessage);
      }

    } else {
      $this->logger->error('Amazon SNS Message event does not contain expected properties.');
      return FALSE;
    }

    return TRUE;
  }
}
