<?php

namespace Drupal\ww_publish;

class Field {

  /**
   * Configuration of the field.
   *
   * @var \Drupal\field\Entity\FieldConfig
   */
  private $fieldConfig;

  /**
   * WoodWing Studio identifier.
   *
   * @var string
   */
  private $contentStationIdentifier;

  /**
   * Metadata of the article received from WoodWing Studio.
   *
   * @var \Drupal\ww_publish\Metadata
   */
  private $articleMetadata;

  /**
   * Article Data.
   *
   * @var Object
   */
  private $articleData;

  /**
   * Prepared files.
   *
   * @var array
   */
  private $preparedFiles;

  /**
   * WoodWing Studio article ID field.
   *
   * @var string
   */
  private $articleIdField;

  /**
   * Configuration of the ww_publish module.
   *
   * @var \Drupal\Core\Config\Config
   */
  private $config;

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  private $logger;

  /**
   * Constructor.
   *
   * @param \Drupal\field\Entity\FieldConfig $field_config
   *   Field configuration.
   * @param string $content_station_identifier
   *   WoodWing Studio identifier.
   * @param \Drupal\ww_publish\Metadata $article_metadata
   *   Article metadata.
   * @param Object $article_data
   *   Article data.
   * @param array $prepared_files
   *   Prepared files.
   * @param string $article_id_field
   *   WoodWing Studio article ID field.
   * @param \Drupal\Core\Config\Config $config
   *   Configuration of the ww_publish module.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   */
  public function __construct($field_config, $content_station_identifier, $article_metadata, $article_data, $prepared_files, $article_id_field, $config, $logger) {
    $this->fieldConfig = $field_config;
    $this->contentStationIdentifier = $content_station_identifier;
    $this->articleMetadata = $article_metadata;
    $this->articleData = $article_data;
    $this->preparedFiles = $prepared_files;
    $this->articleIdField = $article_id_field;
    $this->config = $config;
    $this->logger = $logger;
  }

  /**
   * Get article data.
   *
   * @return array
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function getArticleData() {
    $articleData = [];
    $contentStationIdentifiers = explode('|', $this->contentStationIdentifier);
    switch ($this->fieldConfig->get('field_type')) {
      case 'text_with_summary':
        // Get field value.
        $fieldValue = $this->getComponentContent($contentStationIdentifiers[0]);
        if (is_array($fieldValue)) {
          $articleData['value'] = implode(' ', $fieldValue);
        } else {
          $articleData['value'] = $fieldValue;
        }

        // Add summary, if defined in WoodWing Studio identifier, e.g.: 'body|intro'.
        $noSummary = TRUE;
        if (count($contentStationIdentifiers) === 2) {
          $summary = $this->getComponentContent($contentStationIdentifiers[1]);
          if (is_array($summary)) {
            $articleData['summary'] = implode(' ', $summary);
          } else {
            $articleData['summary'] = $summary;
          }
          $noSummary = $articleData['summary'] === FALSE;
        }

        if ($articleData['value'] === FALSE && $noSummary) {
          $articleData = [];
        } else {
          $articleData['format'] = 'full_html';
        }
        break;

      case 'string':
        $fieldValue = FALSE;
        if (count($contentStationIdentifiers) === 1) {
          $fieldValue = $this->getComponentContent($contentStationIdentifiers[0]);
        }
        if (count($contentStationIdentifiers) === 2) {
          $fieldValue = $this->getComponentContent($contentStationIdentifiers[0], $contentStationIdentifiers[1]);
        }
        if ($fieldValue) {
          if (is_array($fieldValue)) {
            $articleData['value'] = $this->shortenValueToFieldMaxLength(implode(' ', $fieldValue));
          } else {
            $articleData['value'] = $this->shortenValueToFieldMaxLength($fieldValue);
          }
        }
        break;

      case 'entity_reference':
        if ($this->config->get('debug_mode'))
          $this->logger->info('Entity reference!');
        $component = FALSE;
        if ($this->config->get('debug_mode'))
          $this->logger->info('Identifiers: <pre><code>' . print_r($contentStationIdentifiers, TRUE) . '</code></pre>');
        if (count($contentStationIdentifiers) === 2) {
          $component = $this->getComponent($contentStationIdentifiers[0], $contentStationIdentifiers[1]);
          if ($this->config->get('debug_mode'))
            $this->logger->info('Component: <pre><code>' . print_r($component, TRUE) . '</code></pre>');
        }
        $entityType = $this->getEntityType();
        if ($component && $entityType) {
          $referenceId = $this->getReferenceId($entityType, $component);
          if ($this->config->get('debug_mode'))
            $this->logger->info('Reference ID: ' .  $referenceId);
          $articleData['target_id'] = $referenceId;
        }
        break;

      default:
        $fieldValue = $this->getComponentContent($this->contentStationIdentifier);
        if ($fieldValue) {
          if (is_array($fieldValue)) {
            $fieldValue = implode(' ', $fieldValue);
          }
          $articleData['value'] = $this->applyTemplate($fieldValue, 'component', [$this->contentStationIdentifier => '']);
          $articleData['format'] = 'full_html';
        }
        break;
    }
    if (empty($articleData)) {
      return NULL;
    } else {
      return $articleData;
    }
  }

  /**
   * Get the component content.
   *
   * @param string $contentStationIdentifier
   *   WoodWing Studio identifier.
   * @param string $contentProperty
   *   Property in the content object.
   * @param string $textProperty
   *   Property in the object in the text array.
   *
   * @return string|array|bool
   */
  private function getComponentContent($contentStationIdentifier, $contentProperty = 'text', $textProperty = 'insert') {
    $componentContent = [];
    // Find the related component in the article object.
    foreach ($this->articleData->data->content as $contentStationComponent) {
      if (
        ($contentStationComponent->id === $contentStationIdentifier || $contentStationComponent->identifier === $contentStationIdentifier) &&
        property_exists($contentStationComponent, 'content') && property_exists($contentStationComponent->content, $contentProperty)
      ) {
        foreach ($contentStationComponent->content->{$contentProperty} as $text) {
          if (property_exists($text, 'attributes')) {
            $componentContent[] = $this->applyTemplate($text->{$textProperty}, 'attribute', $text->attributes);
          } else {
            $componentContent[] = $text->{$textProperty};
          }
        }
      }
    }
    // If there is only one value in the array, then only the value should be returned.
    if (count($componentContent) === 1) {
      $componentContent = $componentContent[0];
    }

    if (empty($componentContent)) {
      return FALSE;
    } else {
      return $componentContent;
    }
  }

  /**
   * Apply template to the component
   *
   * @param string $text
   *   Text from the component, to which the template(s) should by applied.
   * @param string $type
   *   Type of the template: component, attribute.
   * @param array $attributes
   *   Attributes with data about the templates.
   *
   * @return string
   */
  private function applyTemplate($text, $type, $attributes = NULL) {
    foreach ($attributes as $attribute => $attributeValue) {
      $template = new Template($attribute, $type, $this->logger);
      if ($template->exist()) {
        $text = $template->apply([
          'insert' => $text,
          $attribute => $attributeValue,
        ]);
      }
    }

    return $text;
  }

  /**
   * Get the WoodWing component.
   *
   * @param string $contentStationIdentifier
   *   WoodWing Studio identifier.
   * @param string $contentProperty
   *   Property in the content object.
   *
   * @return Object
   */
  private function getComponent($contentStationIdentifier, $contentProperty) {
    // Find the related component in the article object.
    foreach ($this->articleData->data->content as $contentStationComponent) {
      if ($contentStationComponent->id === $contentStationIdentifier || $contentStationComponent->identifier === $contentStationIdentifier) {
        return $contentStationComponent->content->{$contentProperty};
      }
    }
    return NULL;
  }

  /**
   * Get type of the referenced entity.
   *
   * @return string
   */
  private function getEntityType() {
    $entityType = NULL;
    if (
      $this->fieldConfig->getSetting('handler') === 'default:media' &&
      $this->fieldConfig->getSetting('handler_settings') &&
      count($this->fieldConfig->getSetting('handler_settings')['target_bundles']) === 1
    ) {
      $entityType = reset($this->fieldConfig->getSetting('handler_settings')['target_bundles']);
    }
    return $entityType;
  }

  /**
   * Get reference ID.
   *
   * @param string $entityType
   *   Type of the referenced entity.
   * @param Object $component
   *   Component.
   *
   * @return integer
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function getReferenceId($entityType, $component) {
    switch ($entityType) {
      case 'image':
        $image = new Image($component->id, $this->articleMetadata, $this->preparedFiles, $this->articleIdField, $this->config, $this->logger);
        if ($mediaEntity = $image->getMediaEntity()) {
          return $mediaEntity->id();
        } else {
          $this->logger->error('Image media entity could not be created for the component: <pre><code>' . print_r($component, TRUE) . '</code></pre>');
          return NULL;
        }

        break;
    }

    return NULL;
  }

  /**
   * Return shortened value, if the original value is longer as the field
   * maximal length.
   *
   * @param string $value
   *   Value to be shortened.
   *
   * @return string
   */
  private function shortenValueToFieldMaxLength($value) {
    $fieldMaxLength = $this->fieldConfig->getFieldStorageDefinition()->getSettings()['max_length'];

    if (isset($fieldMaxLength) && is_int($fieldMaxLength) && strlen($value) > $fieldMaxLength) {
      $value = substr($value, 0, $fieldMaxLength);

      // Log the error message, that the value has to be shortened.
      $this->logger->error('The value which should be saved in the field <strong>'
        . $this->fieldConfig->getName() . '</strong> is too long.<br> Maximum length: <strong>'
        . $fieldMaxLength . '</strong>.<br> Shortened value: <i>' . $value
        . '</i>'
      );
    }

    return $value;
  }

}
