<?php

namespace Drupal\ww_publish\Form;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configures WoodWing Studio settings for this site.
 */
class WwPublishSettingsForm extends ConfigFormBase {

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Constructs a DirectoryForm.
   *
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   */
  public function __construct(FileSystemInterface $file_system = NULL) {
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('file_system')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ww_publish_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['ww_publish.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('ww_publish.settings');

    // General form.
    $form['#attributes']['autocomplete'] = 'off';

    $form['content_station'] = [
      '#type' => 'details',
      '#title' => 'WoodWing Studio',
      '#open' => TRUE,
    ];
    $form['content_station']['content_type_field'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Content type field'),
      '#description' => $this->t('WoodWing Studio metadata field name which contains the machine name of the Drupal content type, in which the nodes should be created/updated.'),
      '#default_value' => $config->get('content_type_field'),
      '#required' => TRUE,
    ];

    // Get all users having the permission to create a media.
    $roles = user_role_names(FALSE, 'create media');
    $rids = array_keys($roles);
    $uids = \Drupal::service('entity_type.manager')
      ->getStorage('user')
      ->getQuery()
      ->condition('roles', $rids, 'IN')
      ->execute();
    $users = [];
    foreach ($uids as $uid) {
      $user = User::load($uid);
      $users[$uid] = $user->getAccountName();
    }
    $defaultUser = ($config->get('content_type_field')) ? $config->get('ww_user') : 1;
    $form['content_station']['ww_user'] = [
      '#type' => 'select',
      '#title' => $this->t('WW User'),
      '#description' => $this->t('This user will be used to save WW contents in Drupal.'),
      '#default_value' => $defaultUser,
      '#options' => $users,
      '#required' => TRUE,
    ];

    $form['content_station']['ww_user_field'] = [
      '#type' => 'textfield',
      '#title' => $this->t('WW User field'),
      '#maxlength' => 255,
      '#default_value' => $config->get('ww_user_field'),
      '#description' => $this->t('Field in the user entity used to identify the WoodWing user. E.g.: field_ww_user.'),
    ];

    $form['content_station']['ww_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Path for the WW public folder'),
      '#maxlength' => 255,
      '#default_value' => $config->get('ww_path'),
      '#description' => $this->t('If the path in the file system does not exists, then will be the directory created.'),
      '#required' => TRUE,
    ];

    $form['content_station']['debug_mode'] = [
      '#type' => 'checkbox',
      '#title' => t('Activate the debug mode.'),
      '#description' => $this->t('Do not use the debug mode on the live site.'),
      '#default_value' => $config->get('debug_mode'),
    ];

    $form['content_station']['cron_mode'] = [
      '#type' => 'checkbox',
      '#title' => t('Activate the cron mode.'),
      '#description' => $this->t('In cron mode are the Amazon SNS requests saved in the database and the articles are published, when the next cron job runs. Use the cron mode, when the publishing is too slow, what causes that you have multiple Amazon SNS requests when publishing an article.'),
      '#default_value' => $config->get('cron_mode'),
    ];


    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $path = trim($form_state->getValue('ww_path'));
    $config = $this->config('ww_publish.settings');
    $config->set('content_type_field', trim($form_state->getValue('content_type_field')));
    $config->set('ww_user', trim($form_state->getValue('ww_user')));
    $config->set('ww_user_field', trim($form_state->getValue('ww_user_field')));
    $config->set('ww_path', $path);
    $config->set('debug_mode', $form_state->getValue('debug_mode'));
    $config->set('cron_mode', $form_state->getValue('cron_mode'));
    $config->save();

    // Create the directory if it does not exist.
    $directory_path = 'public://' . $path;
    $this->fileSystem->prepareDirectory($directory_path, FileSystemInterface::CREATE_DIRECTORY);

    parent::submitForm($form, $form_state);
  }

}
