<?php

namespace Drupal\ww_publish;

use Drupal\Core\File\FileSystemInterface;
use Drupal\media\Entity\Media;
use Symfony\Component\DependencyInjection\ContainerInterface;

class Image {

  /**
   * WoodWing Studio image ID.
   *
   * @var integer
   */
  private $wwId;

  /**
   * Metadata of the article received from WoodWing Studio.
   *
   * @var \Drupal\ww_publish\Metadata
   */
  private $articleMetadata;

  /**
   * Prepared files.
   *
   * @var array
   */
  private $preparedFiles;

  /**
   * WoodWing Studio article ID field.
   *
   * @var string
   */
  private $articleIdField;

  /**
   * Configuration of the ww_publish module.
   *
   * @var \Drupal\Core\Config\Config
   */
  private $config;

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  private $logger;

  /**
   * Constructor.
   *
   * @param integer $ww_id
   *   WoodWing Studio image ID.
   * @param \Drupal\ww_publish\Metadata $article_metadata
   *   Article metadata.
   * @param array $prepared_files
   *   Prepared files.
   * @param string $article_id_field
   *   WoodWing Studio article ID field.
   * @param \Drupal\Core\Config\Config $config
   *   Configuration of the ww_publish module.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   */
  public function __construct($ww_id, $article_metadata, $prepared_files, $article_id_field, $config, $logger) {
    $this->wwId = $ww_id;
    $this->articleMetadata = $article_metadata;
    $this->preparedFiles = $prepared_files;
    $this->articleIdField = $article_id_field;
    $this->config = $config;
    $this->logger = $logger;
  }

  /**
   * Get Media entity of the image.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function getMediaEntity() {
    // Find the media entity.
    $imageIds = \Drupal::entityQuery('media')
      ->condition('bundle','image')
      ->condition($this->articleIdField, $this->wwId)
      ->execute();

    if ($this->config->get('debug_mode'))
      $this->logger->info('Article ID Field: ' . $this->articleIdField . ', WW ID: ' . $this->wwId . ', image IDs: <pre><code>' . print_r($imageIds, TRUE) . '</code></pre>');

    if (empty($imageIds)) {
      return $this->createMediaEntity();
    } else {
      return Media::load(reset($imageIds));
    }
  }

  /**
   * Create a new image media entity.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function createMediaEntity() {
    if ($this->preparedFiles['images'][$this->wwId]) {
      $filePath = $this->preparedFiles['images'][$this->wwId];
    } else {
      return NULL;
    }
    $file_data = file_get_contents($filePath);

    // Create the directory if it does not exist.
    $directory_path = 'public://' . $this->config->get('ww_path') . '/' . date('Y-m');
    $fileSystem = \Drupal::service('file_system');
    $fileSystem->prepareDirectory($directory_path, FileSystemInterface::CREATE_DIRECTORY);

    $destination = $directory_path . '/' .  $this->articleMetadata->getFileName($this->wwId, $filePath);
    if ($this->config->get('debug_mode'))
      $this->logger->info('File destination: <pre><code>' . print_r($destination, TRUE) . '</code></pre>');

    $file = file_save_data($file_data, $destination, FileSystemInterface::EXISTS_REPLACE);

    $mediaData = [
      'bundle' => 'image',
      'name' => $file->getFilename(),
      'status' => 1,
      'uid' => $this->articleMetadata->getAuthor()->id(),
      'field_media_image' => [
        'target_id' => $file->id(),
        'alt' => $file->getFilename(),
        'title' => $file->getFilename(),
      ],
      $this->articleIdField => $this->wwId,
    ];
    if ($this->config->get('debug_mode'))
      $this->logger->info('Media data: <pre><code>' . print_r($mediaData, TRUE) . '</code></pre>');
    $media = Media::create($mediaData);
    $media->save();

    return $media;
  }

}
