<?php

namespace Drupal\ww_publish;

use Drupal\Core\Archiver\ArchiverException;
use Drupal\Core\Archiver\ArchiverInterface;
use Drupal\Core\Archiver\Zip as DrupalZip;

class Message {

  /**
   * SNS Message.
   *
   * @var Object
   */
  private $message = null;

  /**
   * Error messages.
   *
   * @var array
   */
  private $errors = array();

  public function __construct($message = null) {
    $this->message = $message;
    $this->errors = array();
  }

  public function getArticleID() {
    if ($this->message) {
      return $this->message->id;
    }

    return false;
  }

  public function getArticleName() {
    if ($this->message) {
      return $this->message->name;
    }

    return false;
  }

  public function getArticleZipName() {
    if ($this->message) {
      try {
        $zipname = basename($this->message->url);
      }
      catch (Exception $e) {
        $this->errors[] = "Problem loading article zipped data, error: $e";
      }
      return $zipname;
    }

    return false;
  }

  public function prepareArticleFiles() {
    $fileSystem = \Drupal::service('file_system');
    // Check tmp dir. Create if not exist.
    $tempDir = \Drupal::service('file_system')->getTempDirectory();
    if (!file_exists($tempDir)) {
      if (!$fileSystem->mkdir($tempDir)) {
        $this->errors[] = "failed to create tmp dir [$tempDir], no further processing";
        return false;
      }
    }
    // Save to disk.
    $zipName = $this->getArticleZipName();
    // Store the ZIP file in the temporary folder.
    $zipNamePath = $tempDir . DIRECTORY_SEPARATOR . $zipName;
    $this->downloadArticleZipToPath($zipNamePath);

    // Prepare the drupal-side.
    $articleDirName = $this->getArticleID() . '-' . $zipName;
    $articleDir = $tempDir . DIRECTORY_SEPARATOR . $articleDirName;
    if (file_exists($articleDir)) {
      $fileSystem->deleteRecursive($articleDir);
    }

    // Unzip the ZIP file to the folder.
    try {
      $zip = new DrupalZip($zipNamePath);
      $zip->extract($articleDir);
    } catch (ArchiverInterface $e) {
      $this->errors[] = $e->getMessage();
      return false;
    } catch (ArchiverException $e) {
      $this->errors[] = $e->getMessage();
      return false;
    }

    if (file_exists($articleDir . '/img')) {
      $images = $this->getImagesFromPath($articleDir . '/img');
    }
    else {
      $images = [];
    }

    return [
      'images' => $images,
      'filesToRemoveAfterPublish' => [
        $zipNamePath,
        $articleDir
      ]
    ];
  }

  /**
   * Download the ZIP file.
   *
   * Get the ZIP file from the path specified in the message and download it to
   * the specified path.
   *
   * @param string $zipPath
   *   Path for the ZIP file.
   *
   * @return bool|false|string|null
   */
  public function downloadArticleZipToPath($zipPath) {
    $zipData = false;
    if ($this->message) {
      try {
        $zipData = file_get_contents($this->message->url);
        file_put_contents($zipPath, $zipData);
      }
      catch (Exception $e) {
        $this->errors[] = "Problem loading article zipped data, error: $e";
        return null;
      }
    }

    return $zipData;
  }

  private function getImagesFromPath($imageFolder) {
    $imagesPath = [];
    $dh = opendir($imageFolder);
    if ($dh !== false) {
      $filename = readdir($dh);
      while ($filename !== false) {
        if ($filename != '.' && $filename != '..' && $filename !== false) {
          $filenameWithoutType = substr($filename, 0, strpos($filename, '.'));
          $imagesPath[$filenameWithoutType] = $imageFolder .'/'.  $filename ;
        }
        $filename = readdir($dh);
      }
    }
    closedir($dh);

    return $imagesPath;
  }

  public function getArticleMetadata() {
    if ($this->message) {
      try {
        $metadata = file_get_contents($this->message->metadataUrl);
      }
      catch (Exception $e) {
        $this->errors[] = "Problem loading article metadata, error: $e";
        return null;
      }
      return json_decode($metadata);
    }

    return false;
  }

  public function getArticleJSON() {
    if ($this->message) {
      try {
        $articleJSON = file_get_contents($this->message->articleJsonUrl);
      }
      catch (Exception $e) {
        $this->errors[] = "Problem loading article JSON, error: $e";
        return null;
      }
      return json_decode($articleJSON);
    }

    return false;
  }

  public function getErrors () {
    if ($this->errors) {
      return $this->errors;
    }
    return false;
  }

}
