<?php

namespace Drupal\ww_publish;

use Drupal\field\Entity\FieldConfig;
use Drupal\paragraphs\Entity\Paragraph;

class ParagraphField {

  /**
   * Paragraph filed configuration.
   *
   * @var \Drupal\entity_reference_revisions\EntityReferenceRevisionsFieldItemList
   */
  private $fieldConfig;

  /**
   * Metadata of the article received from WoodWing Studio.
   *
   * @var \Drupal\ww_publish\Metadata
   */
  private $articleMetadata;

  /**
   * Article Components.
   *
   * @var array
   */
  private $articleComponents;

  /**
   * Prepared files.
   *
   * @var array
   */
  private $preparedFiles;

  /**
   * WoodWing Studio article ID field.
   *
   * @var string
   */
  private $articleIdField;

  /**
   * Configuration of the ww_publish module.
   *
   * @var \Drupal\Core\Config\Config
   */
  private $config;

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  private $logger;

  /**
   * Constructor.
   *
   * @param \Drupal\entity_reference_revisions\EntityReferenceRevisionsFieldItemList $field_config
   *   Field config.
   * @param \Drupal\ww_publish\Metadata $article_metadata
   *   Article components.
   * @param array $article_components
   *   Article data.
   * @param array $prepared_files
   *   Prepared files.
   * @param string $article_id_field
   * @param \Drupal\Core\Config\Config $config
   *   Configuration of the ww_publish module.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   */
  public function __construct($field_config, $article_metadata, $article_components, $prepared_files, $article_id_field, $config, $logger) {
    $this->fieldConfig = $field_config;
    $this->articleMetadata = $article_metadata;
    $this->articleComponents = $article_components;
    $this->preparedFiles = $prepared_files;
    $this->articleIdField = $article_id_field;
    $this->config = $config;
    $this->logger = $logger;
  }

  /**
   * Get article data.
   *
   * @return array
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function getArticleData() {

    $identifiers = $this->getIdentifiers();
    $articleData = [];

    foreach ($this->articleComponents as $contentStationComponent) {
      $identifierKeys = array_keys(
        array_column($identifiers, 'first_identifier'),
        $contentStationComponent->identifier
      );

      if (!empty($identifierKeys)) {
        $identifierData = [];
        foreach ($identifierKeys as $identifierKey) {
          $identifierData[] = $identifiers[$identifierKey];
        }
        $paragraph = $this->createParagraph($contentStationComponent, $identifierData);

        if ($paragraph) {
          $articleData[] = [
            'target_id' => $paragraph->id(),
            'target_revision_id' => $paragraph->getRevisionId(),
          ];
        }
      }
    }

    if ($this->config->get('debug_mode'))
      $this->logger->info('Paragraph article data: <pre><code>' . print_r($articleData, TRUE) . '</code></pre>');
    return $articleData;
  }

  /**
   * Get content of the component.
   *
   * @param Object $contentStationComponent
   *   WoodWing Studio componennt.
   * @param array $identifiers
   *   The array items are arrays with the following keys: 'identifier',
   *   'first_identifier', 'last_identifier', 'paragraph_type', 'field_name',
   *   'field_type', 'field_config'.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  private function createParagraph($contentStationComponent, $identifiers) {

    $paragraphData = [
      'type' => $identifiers[0]['paragraph_type'],
    ];

    $createParagraph = FALSE;

    foreach ($identifiers as $identifier) {
      $contentStationIdentifiers = explode('|', $identifier['identifier']);

      switch ($identifier['field_type']) {
        case 'entity_reference':
          $entityType = $this->getEntityType($identifier['field_config']);
          $referenceId = $this->getReferenceId($entityType, $contentStationComponent);
          if ($referenceId) {
            $paragraphData[$identifier['field_name']] = [
              'target_id' => $referenceId,
            ];
            $createParagraph = TRUE;
          }
          break;

        case 'entity_reference_revisions':
          if ($this->config->get('debug_mode')) {
            $this->logger->info('WoodWing Studio component containers for ' . $contentStationComponent->identifier . ': <pre><code>' . print_r($contentStationComponent->containers, TRUE) . '</code></pre>');
            $this->logger->info('Last identifier: ' . $identifier['last_identifier']);
          }
          $paragraphField = new ParagraphField($identifier['field_config'], $this->articleMetadata, $contentStationComponent->containers->{$identifier['last_identifier']}, $this->preparedFiles, $this->articleIdField, $this->config, $this->logger);
          $articleData = $paragraphField->getArticleData();
          if ($articleData) {
            $paragraphData[$identifier['field_name']] = $articleData;
            $createParagraph = TRUE;
          }
          break;

        case 'string':
          $componentContent = FALSE;
          if (count($contentStationIdentifiers) === 1) {
            $componentContent = $this->getComponentContent($contentStationComponent);
          }
          if (count($contentStationIdentifiers) === 2) {
            $componentContent = $this->getComponentContent($contentStationComponent, $contentStationIdentifiers[1]);
          }

          if ($componentContent) {
            $paragraphData[$identifier['field_name']] = $componentContent;
            $createParagraph = TRUE;
          }
          break;

        case 'link':
          $uri = $this->getComponentContent($contentStationComponent);
          if ($uri) {
            $paragraphData[$identifier['field_name']] = [
              'uri' => $uri,
            ];
            $createParagraph = TRUE;
          }
          break;

        default:
          // Get content from the text property.
          $content = '';
          if (property_exists($contentStationComponent, 'content') && property_exists($contentStationComponent->content, 'text')) {
            foreach ($contentStationComponent->content->text as $text) {
              $attributeContent = $text->insert;
              if (property_exists($text, 'attributes')) {
                foreach ($text->attributes as $attribute => $attributeValue) {
                  $template = new Template($attribute, 'attribute', $this->logger);
                  if ($template->exist()) {
                    $attributeContent = $template->apply([
                      'insert' => $attributeContent,
                      $attribute => $attributeValue,
                    ]);
                  }
                }
              }
              $content .= $attributeContent;
            }
          }

          if (!empty($content)) {
            // Apply a template on the content.
            $template = new Template($identifier['identifier'], 'component', $this->logger);
            if ($template->exist()) {
              $content = $template->apply(['insert' => $content]);
            }

            // Create a new paragraph.
            $paragraphData[$identifier['field_name']] = [
              'value' => $content,
              'format' => 'full_html',
            ];

            $createParagraph = TRUE;
          }
      }
    }

    if ($createParagraph) {
      if ($this->config->get('debug_mode'))
        $this->logger->info('Paragraph data: <pre><code>' . print_r($paragraphData, TRUE) . '</code></pre>');

      $paragraph = Paragraph::create($paragraphData);
      $paragraph->save();

      return $paragraph;
    } else {
      return NULL;
    }
  }

  /**
   * Get type of the referenced entity.
   *
   * @param \Drupal\field\Entity\FieldConfig $fieldConfig
   *   Field configuration.
   *
   * @return string
   */
  private function getEntityType($fieldConfig) {
    $entityType = NULL;
    if (
      $fieldConfig->getSetting('handler') === 'default:media' &&
      $fieldConfig->getSetting('handler_settings') &&
      count($fieldConfig->getSetting('handler_settings')['target_bundles']) === 1
    ) {
      $entityType = reset($fieldConfig->getSetting('handler_settings')['target_bundles']);
    }
    return $entityType;
  }

  /**
   * Get reference ID.
   *
   * @param string $entityType
   *   Type of the referenced entity.
   * @param Object $component
   *   Component.
   *
   * @return integer
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function getReferenceId($entityType, $component) {
    switch ($entityType) {
      case 'image':
        $image = new Image($component->content->image->id, $this->articleMetadata, $this->preparedFiles, $this->articleIdField, $this->config, $this->logger);
        if ($mediaEntity = $image->getMediaEntity()) {
          return $mediaEntity->id();
        } else {
          $this->logger->error('Image media entity could not be created for the component: <pre><code>' . print_r($component, TRUE) . '</code></pre>');
          return NULL;
        }

        break;
    }

    return NULL;
  }

  /**
   * Get the component content.
   *
   * @param Object $contentStationComponent
   *   WoodWing Studio component, from which the content should be extracted.
   * @param string $contentStationIdentifier
   *   WoodWing Studio identifier.
   * @param string $contentProperty
   *   Property in the content object.
   * @param string $textProperty
   *   Property in the object in the text array.
   *
   * @return string|array
   */
  private function getComponentContent($contentStationComponent, $contentProperty = 'text', $textProperty = 'insert') {
    $componentContent = [];
    foreach ($contentStationComponent->content->{$contentProperty} as $text) {
      $componentContent[] = $text->{$textProperty};
    }
    // If there is only one value in the array, then only the value should be returned.
    if (count($componentContent) === 1) {
      $componentContent = $componentContent[0];
    }
    return $componentContent;
  }

  /**
   * Get all defined WoodWing Studio identifiers.
   *
   * All identifiers from fields in all enabled paragraph types is returned as
   * an array, of arrays with the following keys: identifier, paragraph_type,
   * field_name.
   *
   * @return array
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  public function getIdentifiers() {
    $identifiers = [];
    // Get all enabled paragraph types.
    $enabledParagraphTypes = $this->fieldConfig->get('settings')['handler_settings']['target_bundles'];
    foreach ($enabledParagraphTypes as $type) {
      $fields = \Drupal::service('entity.manager')->getFieldDefinitions('paragraph', $type);
      foreach ($fields as $field_name => $fieldConfig) {
        if ($fieldConfig instanceof FieldConfig && $fieldConfig->get('entity_type') === 'paragraph') {
          $identifier = $fieldConfig->getThirdPartySetting('ww_publish', 'content_station_identifier');
          $paragraph_type = $fieldConfig->get("bundle");
          $explodedIdentifier = explode('|', $identifier);
          $lastIdentifier = end($explodedIdentifier);
          $identifiers[] = [
            'identifier' => $identifier,
            'first_identifier' => strtok($identifier, '|'),
            'last_identifier' => $lastIdentifier,
            'paragraph_type' => $paragraph_type,
            'field_name' => $field_name,
            'field_type' => $fieldConfig->getType(),
            'field_config' => $fieldConfig,
          ];
        }
      }
    }
    return $identifiers;
  }
}
